==> selene-0.2.0.tar.gz: lib/src/models/message.dart <==
part of selene;

/// A message sent in a channel in Discord.
class Message extends Entity {
  /// Creates a new [Message].
  Message(Client client) : super(client);

  /// The ID of the channel this message belongs to.
  int channelId;

  /// The author of this message.
  ///
  /// Will be null if it is sent by a webhook. (Use `webhook` instead)
  User get author => client.cache.users[_authorId];

  /// The author of this message as a [GuildMember] object.
  ///
  /// Will be null if it is sent by a webhook. (Use `webhook` instead)
  GuildMember get member {
    if (isNotInGuild) return null;
    return (channel as GuildTextChannel).guild.members[_authorId];
  }

  /// Internal ID of the author.
  int _authorId;

  // TODO: Webhooks

  /// The [TextChannel] to which this message was sent.
  TextChannel channel;

  /// Whether this message was sent in a [GuildTextChannel].
  bool get isInGuild => (channel is GuildTextChannel);

  /// Whether this message was not sent in a [GuildTextChannel].
  bool get isNotInGuild => !isInGuild;

  /// The content of this message.
  String content;

  /// The timestamp of this message.
  DateTime timestamp;

  /// The timestamp of when this message was edited.
  ///
  /// Null if this message was never edited.
  DateTime editedTimestamp;

  /// Whether this message is TTS (Text-To-Speech).
  bool isTTS;

  /// Whether this message is not TTS (Text-To-Speech).
  bool get isNotTTS => !isTTS;

  /// Whether this message mentions @everyone.
  bool mentionsEveryone;

  /// Whether this message does not mention @everyone.
  bool get doesNotMentionEveryone => !mentionsEveryone;

  /// A list of all users mentioned in this message.
  List<User> mentions = [];

  // TODO: Roles mentioned

  // TODO: Attachments

  // TODO: Embeds

  /// Reactions that have been applied to this message.
  List<Reaction> reactions = [];

  // TODO: Nonce?

  /// Whether this message was pinned.
  bool isPinned;

  /// Whether this message was not pinned.
  bool get isNotPinned => !isPinned;

  /// The type of message.
  MessageType type;

  /// The ID of the webhook, if this message was created by a webhook.
  int webhookId;

  /// Whether this message was created by a webhook.
  bool get createdByWebhook => webhookId != null;

  /// Whether this message was not created by a webhook.
  bool get notCreatedByWebhook => !createdByWebhook;

  /// Deletes this message.
  ///
  /// Client must either have `MANAGE_MESSAGES`, or be the sender of this message.
  Future delete() => channel.deleteMessage(this);

  /// Edits this message.
  ///
  /// Client must be the sender of this message.
  Future edit(void update(MessageUpdate update)) =>
      channel.editMessage(this, update);

  @override
  void _update(Map<String, dynamic> model) {
    super._update(model);

    channelId = int.tryParse((model['channel_id'] ?? channelId) ?? '');
    content = model['content'] ?? content;

    if (model['author'] != null) {
      if (model['webhook_id'] == null) {
        // Not a webhook
        _authorId = int.tryParse(model['author']['id'] ?? '');
        if (!client.cache.users.containsKey(_authorId)) {
          var newUser = new User(client);
          newUser._update(model['author']);
          client.cache.users[newUser.id] = newUser;
        }
      } else {
        webhookId = int.tryParse(model['webhook_id'] ?? '');
      }
    }

    if (model['timestamp'] != null) {
      timestamp = DateTime.parse(model['timestamp']);
    }
    if (model['edited_timestamp'] != null) {
      editedTimestamp = DateTime.parse(model['edited_timestamp']);
    }

    if (channel == null) {
      channel = client.cache.resolveChannel(channelId) as TextChannel;
    }

    isTTS = model['tts'] ?? isTTS;
    mentionsEveryone = model['mention_everyone'] ?? mentionsEveryone;

    for (var jsonMentionee in model['mentions'] ?? []) {
      var mentionee = new User(client);
      mentionee._update(jsonMentionee);
      mentions.add(mentionee);
    }

    for (var jsonReaction in model['reactions'] ?? []) {
      var reaction = new Reaction(client);
      reaction._update(jsonReaction);
      reactions.add(reaction);
    }

    isPinned = model['pinned'] ?? isPinned;
    webhookId = model['webhook_id'] ?? webhookId;
    switch (model['type']) {
      case 0:
        type = MessageType.Default;
        break;
      case 1:
        type = MessageType.RecipientAdd;
        break;
      case 2:
        type = MessageType.RecipientRemove;
        break;
      case 3:
        type = MessageType.Call;
        break;
      case 4:
        type = MessageType.ChannelNameChange;
        break;
      case 5:
        type = MessageType.ChannelIconChange;
        break;
      case 6:
        type = MessageType.ChannelPinnedMessage;
        break;
      case 7:
        type = MessageType.GuildMemberJoin;
        break;
    }
  }
}

/// A type of message.
enum MessageType {
  Default,
  RecipientAdd,
  RecipientRemove,
  Call,
  ChannelNameChange,
  ChannelIconChange,
  ChannelPinnedMessage,
  GuildMemberJoin
}

/// A set of properties for updating a message.
class MessageUpdate {
  String content;
  Map<String, dynamic> embed;

  Map<String, dynamic> _build() {
    var map = <String, dynamic>{};
    if (content != null) map['content'] = content;
    if (embed != null) map['embed'] = embed;
    return map;
  }
}

==> selene-0.3.0.tar.gz: lib/src/models/message.dart <==
part of selene;

/// A message sent in a channel in Discord.
class Message extends SnowflakeEntity {
  Message._create(Map<String, dynamic> model, Client client)
      : super(model, client) {
    channelId = parseInt(model['channel_id']);

    if (model['author'] != null) {
      if (model['webhook_id'] == null) {
        author = new User._cachedOrNew(model['author'], client);
        if (channel is GuildTextChannel) {
          member = (channel as GuildTextChannel).guild.members[author.id];
        }
      } else {
        webhookId = int.tryParse(model['webhook_id'] ?? '');
      }
    }

    if (model['timestamp'] != null) {
      timestamp = DateTime.parse(model['timestamp']);
    }

    if (channel == null) {
      channel = client.state.resolveChannel(channelId) as TextChannel;
    }

    switch (model['type']) {
      case 0:
        type = MessageType.Default;
        break;
      case 1:
        type = MessageType.RecipientAdd;
        break;
      case 2:
        type = MessageType.RecipientRemove;
        break;
      case 3:
        type = MessageType.Call;
        break;
      case 4:
        type = MessageType.ChannelNameChange;
        break;
      case 5:
        type = MessageType.ChannelIconChange;
        break;
      case 6:
        type = MessageType.ChannelPinnedMessage;
        break;
      case 7:
        type = MessageType.GuildMemberJoin;
        break;
    }

    if (channel.messages.length >=
        client.options.messageCacheMaxSize) {
      channel.messages.remove(
          channel.messages.entries.first);
    }

    channel.messages[id] = this;

    _update(model);
  }

  void _parseMentions(List<dynamic> mentions) {
    for (var jsonMentionee in mentions ?? []) {
      if (!this.mentions.any((u) => u.id == parseInt(jsonMentionee['id']))) this
          .mentions.add(new User._cached(jsonMentionee, client));
    }
  }

  void _update(Map<String, dynamic> model) {
    content = model['content'] ?? content;

    if (model['edited_timestamp'] != null) {
      editedTimestamp = DateTime.parse(model['edited_timestamp']);
    }

    isTTS = model['tts'] ?? isTTS;
    mentionsEveryone = model['mention_everyone'] ?? mentionsEveryone;

    _parseMentions(model['mentions']);

    isPinned = model['pinned'] ?? isPinned;
  }

  /// The ID of the channel this message belongs to.
  int channelId;

  /// The author of this message.
  ///
  /// Will be null if it is sent by a webhook. (Use `webhook` instead)
  User author;

  /// The author of this message as a [GuildMember] object.
  ///
  /// Will be null if it is sent by a webhook, or in a DM. (Use `webhook` instead)
  GuildMember member;

  // TODO: Webhooks

  /// The [TextChannel] to which this message was sent.
  TextChannel channel;

  /// The content of this message.
  String content;

  /// The timestamp of this message.
  DateTime timestamp;

  /// The timestamp of when this message was edited.
  ///
  /// Null if this message was never edited.
  DateTime editedTimestamp;

  /// Whether this message is TTS (Text-To-Speech).
  bool isTTS;

  /// Whether this message is not TTS (Text-To-Speech).
  bool get isNotTTS => !isTTS;

  /// Whether this message mentions @everyone.
  bool mentionsEveryone;

  /// Whether this message does not mention @everyone.
  bool get doesNotMentionEveryone => !mentionsEveryone;

  /// A list of all users mentioned in this message.
  List<User> mentions = [];

  // TODO: Roles mentioned

  // TODO: Attachments

  // TODO: Embeds

  // TODO: Nonce?

  // TODO: Reactions

  /// Whether this message was pinned.
  bool isPinned;

  /// Whether this message was not pinned.
  bool get isNotPinned => !isPinned;

  /// The type of message.
  MessageType type;

  /// The ID of the webhook, if this message was created by a webhook.
  int webhookId;

  /// Whether this message was created by a webhook.
  bool get createdByWebhook => webhookId != null;

  /// Whether this message was not created by a webhook.
  bool get notCreatedByWebhook => !createdByWebhook;

  /// Deletes this message.
  ///
  /// Client must either have `MANAGE_MESSAGES`, or be the sender of this message.
  Future delete() => channel.deleteMessage(this);

  /// Edits this message.
  ///
  /// Client must be the sender of this message.
  Future edit({String content, Map<String, dynamic> embed}) =>
      channel.editMessage(this, content: content, embed: embed);
}

/// A type of message.
enum MessageType {
  Default,
  RecipientAdd,
  RecipientRemove,
  Call,
  ChannelNameChange,
  ChannelIconChange,
  ChannelPinnedMessage,
  GuildMemberJoin
}

/// A set of properties for updating a message.
class MessageUpdate {
  String content;
  Map<String, dynamic> embed;

  Map<String, dynamic> _build() {
    var map = <String, dynamic>{};
    if (content != null) map['content'] = content;
    if (embed != null) map['embed'] = embed;
    return map;
  }
}
